import { MappingType } from "../types/enum";
export declare const EXIST_TEXT = "exist";
export declare const BELONG_TEXT = "belong";
export type SuggestConfig = {
    /**
     * 是否开启debug模式
     */
    debug?: boolean;
    /**
     * 默认字段
     */
    mapping: FieldMapping[] | (() => Promise<FieldMapping[]>);
    /**
     * 候选词对应的 element
     */
    wordElement: () => HTMLElement[] | NodeListOf<HTMLElement>;
    /**
     * 查询搜索建议的接口
     */
    fetchQuery: (params: FetchParams, fetchOptions?: FetchOptions) => Promise<FieldMapping[]>;
    onUpdate?: (info: FieldMapping[]) => void;
};
export type FieldMapping = {
    title: string;
    mappings: MappingItem[];
    render?: (item: MappingItem) => string;
};
/**
 * 标准格式的Mapping
 */
export type MappingItem = {
    name: string;
    value: string;
    type?: MappingFieldType;
    match?: string;
    [key: string]: any;
};
/**
 * 检索语句 字段根据类型分类
 */
export declare enum MappingFieldType {
    IP = "ip",
    DATE = "date",
    VALUE = "value",
    STRING = "string"
}
export type FetchParams = {
    /**
     * 运算符 ： = > < >= <= != exist belong
     */
    operator: string;
    /**
     * 字段
     */
    field: MappingItem;
    /**
     * 用户正在输入的内容
     */
    value: string;
};
export type FetchOptions = {
    canalSignal?: AbortSignal;
};
export declare enum MappingListCate {
    ASSI = "assi",
    ALARM = "alarm",
    LOG = "log"
}
export declare enum MappingModule {
    SEARCH = "search",
    BI = "bi"
}
export type MappingResult = {
    [key: string]: {
        index: string;
        time_field: string;
        mappings: {
            [key: string]: {
                label: string;
                type: MappingFieldType;
            };
        };
        modules: {
            [key in MappingModule]: {
                title: string;
                mappings: string[];
            }[];
        };
    };
};
export declare class Suggest {
    allMappings: Map<MappingType, FieldMapping[]>;
    candiDateList: FieldMapping[];
    private activeIndex;
    private loadingKey;
    _lastTimeStamp: number;
    private config;
    private cacheMap;
    private signalMap;
    constructor(_config: SuggestConfig);
    private debugLog;
    get wordElements(): NodeListOf<HTMLElement> | HTMLElement[];
    /**
     * 当前待选的所有mappings
     * @types FieldMapping[]
     */
    get wordList(): MappingItem[];
    get showList(): FieldMapping[];
    /**
     * 当前选中的元素
     */
    get activeItem(): MappingItem | null;
    /**
     * 当前是否正在loading
     */
    get loading(): boolean;
    /**
     * 初始化数据
     */
    initData(): void;
    initFieldMapping(): void;
    /**
     * 将纯文本转换成格式化之后的数组
     * @param text
     * @example 'source_ip = 1.1.1.1 and dest_ip exist    '
     * ["source_ip","=","1.1.1.1","and","dest_ip","exist"]
     * @returns string
     */
    parseToStringArray(text: string): string[];
    /**
     * 传入一个非空字符串，返回对应的类型,匹配不到时返回 MappingType.InputValue
     * @param text 要解析的字符串
     * @return {MappingType}
     */
    queryTypeByText(text: string): MappingType;
    /**
     * 根据当前字符串，推断下一个字符串的类型
     * 如果是undefined,或者null，或者空字符串。那么返回MappingField
     * @param { string } text
     * @returns {MappingType}
     */
    queryNextType(text: string): MappingType;
    /**
     * 设置显示的列表
     * @param list
     */
    setCandidateList(list: FieldMapping[]): void;
    /**
     * 根据传入的字段，返回对应的mapping。如果没有找到，返回默认值
     * @param text
     * @returns
     */
    queryMappingFieldInfo(text: string): MappingItem;
    /**
     * 过滤搜索建议，返回光标前最后一个有效字段的类型
     * @param text
     * @param options
     * @example "source_ip = 123" return MappingType.InputValue
     * @example "source_ip = 123 an" return MappingType.Logical
     * @example "source_i" return MappingType.MappingField
     * @returns
     */
    filter(text: string, _options?: {
        timeStamp?: number;
    }): MappingType;
    /**
     *  flag :true/ false  开始/结束
     *  uid: 用于标识loading的唯一性
     */
    setLoading(status: boolean, uid?: number | string): void;
    /**
     * 解析纯文本字符串，返回对应的formatter 数组
     * @params {string} text
     * @example "source_ip = 123" return [{formatter: Formatter, value: "source_ip"}, {formatter: Formatter, value: "="}, {formatter: Formatter, value: "123"}]
     * @return [{formatter: Formatter, value: string}]
     */
    parseToFormatter(text: string): {
        formatter: import("@textbus/core").Formatter<import("@textbus/core").FormatValue> | null;
        value: string;
    }[];
    /**
     * 添加AbortCancel 到signalSet
     * @returns { signal: AbortSignal, timeStamp: number}
     */
    addToSignalSet(): {
        signal: AbortSignal;
        timeStamp: number;
    };
    /**
     * 取消当前请求之前的所有请求(包括当前)
     * @param timeStamp
     */
    cancelHandleSignal(timeStamp: number): void;
    /**
     * 设置字段mapping
     * @param mappings
     */
    setMapping(mappingType: MappingType | string, mappings: FieldMapping[]): void;
    /**
     * 移除搜索建议的缓存值。
     *
     */
    removeCache(): void;
    /**
     * 选中第一个
     * @returns
     */
    focus(): void;
    /**
     * 搜索建议失去焦点
     * @returns
     */
    blur(): boolean;
    /**
     * 当前选中值向后移动一个，如果是最后一个，则选中第一个
     * @returns
     */
    toNext(): false | undefined;
    /**
     * 当前选中值向前一个，如果是第一个，则失去焦点
     * @returns {boolean}
     */
    toPrevious(): boolean;
    /**
     * arrowDown
     * 选择到下一行最近的一个元素,如果已经处于最后一行，选择最后一个元素
     * @returns
     */
    toNextLine(): boolean | void;
    /**
     * arrowUP
     * 按下时，如果在第一行，则失去焦点，返回true
     * @returns {boolean} 是否应该返回焦点，即在第一行时按下了上键
     */
    toPreviousLine(): boolean;
}
